@@ -57,6 +57,7 @@
 #include "log.h"
 #include "cfg.h"
 #include "attmap.h"
+#include "common/expr.h"
 
 struct ldap_config *nslcd_cfg=NULL;
 
@@ -75,6 +76,11 @@ struct ldap_config *nslcd_cfg=NULL;
     exit(EXIT_FAILURE); \
   }
 
+/* prototype for parse_validnames_statement() because it is used in
+   cfg_defaults() */
+static void parse_validnames_statement(const char *filename,int lnr,
+                   const char *keyword,char *line,struct ldap_config *cfg);
+
 /* set the configuration information to the defaults */
 static void cfg_defaults(struct ldap_config *cfg)
 {
@@ -97,6 +103,7 @@ static void cfg_defaults(struct ldap_config *cfg)
   cfg->ldc_binddn=NULL;
   cfg->ldc_bindpw=NULL;
   cfg->ldc_rootpwmoddn=NULL;
+  cfg->ldc_rootpwmodpw=NULL;
   cfg->ldc_sasl_mech=NULL;
   cfg->ldc_sasl_realm=NULL;
   cfg->ldc_sasl_authcid=NULL;
@@ -119,6 +126,9 @@ static void cfg_defaults(struct ldap_config *cfg)
   cfg->ldc_pagesize=0;
   cfg->ldc_nss_initgroups_ignoreusers=NULL;
   cfg->ldc_pam_authz_search=NULL;
+  cfg->ldc_nss_min_uid=0;
+  parse_validnames_statement(__FILE__,__LINE__,"",
+                "/^[a-z0-9._@$][a-z0-9._@$ \\~-]*[a-z0-9._@$~-]$/i",cfg);
 }
 
 /* simple strdup wrapper */
@@ -158,74 +168,36 @@ static void add_uri(const char *filename,int lnr,
   cfg->ldc_uris[i].uri=xstrdup(uri);
 }
 
-#ifndef HOST_NAME_MAX
-#define HOST_NAME_MAX 255
-#endif /* not HOST_NAME_MAX */
-
 #ifdef HAVE_LDAP_DOMAIN2HOSTLIST
 /* return the domain name of the current host
    the returned string must be freed by caller */
-static char *cfg_getdomainname(const char *filename,int lnr)
+static const char *cfg_getdomainname(const char *filename,int lnr)
 {
-  char hostname[HOST_NAME_MAX+1],*domain;
-  int hostnamelen;
-  int i;
-  struct hostent *host=NULL;
-  /* get system hostname */
-  if (gethostname(hostname,sizeof(hostname))<0)
-  {
-    log_log(LOG_ERR,"%s:%d: gethostname() failed: %s",filename,lnr,strerror(errno));
-    exit (EXIT_FAILURE);
-  }
-  hostnamelen=strlen(hostname);
-  /* lookup hostent */
-  host=gethostbyname(hostname);
-  if (host==NULL)
-  {
-    log_log(LOG_ERR,"%s:%d: gethostbyname(%s): %s",filename,lnr,hostname,hstrerror(h_errno));
-    exit(EXIT_FAILURE);
-  }
-  /* check h_name for fqdn starting with our hostname */
-  if ((strncasecmp(hostname,host->h_name,hostnamelen)==0)&&
-      (host->h_name[hostnamelen]=='.')&&
-      (host->h_name[hostnamelen+1]!='\0'))
-    return strdup(host->h_name+hostnamelen+1);
-  /* also check h_aliases */
-  for (i=0;host->h_aliases[i]!=NULL;i++)
-  {
-    if ((strncasecmp(hostname,host->h_aliases[i],hostnamelen)==0)&&
-        (host->h_aliases[i][hostnamelen]=='.')&&
-        (host->h_aliases[i][hostnamelen+1]!='\0'))
-      return strdup(host->h_aliases[i]+hostnamelen+1);
-  }
-  /* fall back to any domain part in h_name */
-  if (((domain=strchr(host->h_name,'.'))!=NULL)&&
-      (domain[1]!='\0'))
-    return strdup(domain+1);
-  /* also check h_aliases */
-  for (i=0;host->h_aliases[i]!=NULL;i++)
-  {
-    if (((domain=strchr(host->h_aliases[i],'.'))!=NULL)&&
-        (domain[1]!='\0'))
-      return strdup(domain+1);
-  }
-  /* we've tried everything now */
-  log_log(LOG_ERR,"%s:%d: unable to determinate a domainname for hostname %s",
-          filename,lnr,hostname);
+  const char *fqdn,*domain;
+  fqdn=getfqdn();
+  if ((fqdn!=NULL)&&((domain=strchr(fqdn,'.'))!=NULL)&&(domain[1]!='\0'))
+    return domain+1;
+  log_log(LOG_ERR,"%s:%d: unable to determinate a domain name",
+          filename,lnr);
   exit(EXIT_FAILURE);
 }
 
 /* add URIs by doing DNS queries for SRV records */
 static void add_uris_from_dns(const char *filename,int lnr,
-                              struct ldap_config *cfg)
+                              struct ldap_config *cfg,
+                              const char *domain)
 {
-  int ret=0;
-  char *domain;
+  int rc;
   char *hostlist=NULL,*nxt;
   char buf[HOST_NAME_MAX+sizeof("ldap://")];
-  domain=cfg_getdomainname(filename,lnr);
-  ret=ldap_domain2hostlist(domain,&hostlist);
-  /* FIXME: have better error handling */
+  log_log(LOG_DEBUG,"query %s for SRV records",domain);
+  rc=ldap_domain2hostlist(domain,&hostlist);
+  if (rc!=LDAP_SUCCESS)
+  {
+    log_log(LOG_ERR,"%s:%d: no servers found in DNS zone %s: %s",
+            filename,lnr,domain,ldap_err2string(rc));
+    exit(EXIT_FAILURE);
+  }
   if ((hostlist==NULL)||(*hostlist=='\0'))
   {
     log_log(LOG_ERR,"%s:%d: no servers found in DNS zone %s",filename,lnr,domain);
@@ -249,7 +221,6 @@ static void add_uris_from_dns(const char *filename,int lnr,
     /* get next entry from list */
     hostlist=nxt;
   }
-  free(domain);
 }
 #endif /* HAVE_LDAP_DOMAIN2HOSTLIST */
 
@@ -377,6 +348,28 @@ static inline void check_argumentcount(const char *filename,int lnr,
   }
 }
 
+/* check that the file is not world readable */
+static void check_permissions(const char *filename,const char *keyword)
+{
+  struct stat sb;
+  /* get file status */
+  if (stat(filename,&sb))
+  {
+    log_log(LOG_ERR,"cannot stat() %s: %s",filename,strerror(errno));
+    exit(EXIT_FAILURE);
+  }
+  /* check permissions */
+  if ((sb.st_mode&0007)!=0)
+  {
+    if (keyword!=NULL)
+      log_log(LOG_ERR,"%s: file should not be world readable if %s is set",
+              filename, keyword);
+    else
+      log_log(LOG_ERR,"%s: file should not be world readable",filename);
+    exit(EXIT_FAILURE);
+  }
+}
+
 static void get_int(const char *filename,int lnr,
                     const char *keyword,char **line,
                     int *var)
@@ -442,8 +435,9 @@ static void get_uid(const char *filename,int lnr,
   char *tmp;
   check_argumentcount(filename,lnr,keyword,get_token(line,token,sizeof(token))!=NULL);
   /* check if it is a valid numerical uid */
-  *var=(uid_t)strtol(token,&tmp,0);
-  if ((*token!='\0')&&(*tmp=='\0'))
+  errno=0;
+  *var=strtouid(token,&tmp,10);
+  if ((*token!='\0')&&(*tmp=='\0')&&(errno==0))
     return;
   /* find by name */
   pwent=getpwnam(token);
@@ -467,8 +461,9 @@ static void get_gid(const char *filename,int lnr,
   char *tmp;
   check_argumentcount(filename,lnr,keyword,get_token(line,token,sizeof(token))!=NULL);
   /* check if it is a valid numerical gid */
-  *var=(gid_t)strtol(token,&tmp,0);
-  if ((*token!='\0')&&(*tmp=='\0'))
+  errno=0;
+  *var=strtogid(token,&tmp,10);
+  if ((*token!='\0')&&(*tmp=='\0')&&(errno==0))
     return;
   /* find by name */
   grent=getgrnam(token);
@@ -565,7 +560,7 @@ static void set_base(const char *filename,int lnr,
                      const char *value,const char **var)
 {
 #ifdef HAVE_LDAP_DOMAIN2DN
-  char *domain = NULL;
+  const char *domain=NULL;
   char *domaindn=NULL;
 #endif /* HAVE_LDAP_DOMAIN2DN */
   /* if the base is "DOMAIN" use the domain name */
@@ -574,7 +569,6 @@ static void set_base(const char *filename,int lnr,
 #ifdef HAVE_LDAP_DOMAIN2DN
     domain=cfg_getdomainname(filename,lnr);
     ldap_domain2dn(domain,&domaindn);
-    free(domain);
     log_log(LOG_DEBUG,"set_base(): setting base to %s from domain",domaindn);
     value=domaindn;
 #else /* not HAVE_LDAP_DOMAIN2DN */
@@ -586,6 +580,52 @@ static void set_base(const char *filename,int lnr,
   *var=xstrdup(value);
 }
 
+/* parse the validnames statement */
+static void parse_validnames_statement(const char *filename,int lnr,
+                   const char *keyword,char *line,struct ldap_config *cfg)
+{
+  char *value;
+  int i,l;
+  int flags=REG_EXTENDED|REG_NOSUB;
+  /* the rest of the line should be a regular expression */
+  get_restdup(filename,lnr,keyword,&line,&value);
+  /* check formatting and update flags */
+  if (value[0]!='/')
+  {
+    log_log(LOG_ERR,"%s:%d: regular expression incorrectly delimited",filename,lnr);
+    exit(EXIT_FAILURE);
+  }
+  l=strlen(value);
+  if (value[l-1]=='i')
+  {
+    value[l-1]='\0';
+    l--;
+    flags|=REG_ICASE;
+  }
+  if (value[l-1]!='/')
+  {
+    log_log(LOG_ERR,"%s:%d: regular expression incorrectly delimited",filename,lnr);
+    exit(EXIT_FAILURE);
+  }
+  value[l-1]='\0';
+  /* compile the regular expression */
+  if ((i=regcomp(&cfg->validnames,value+1,flags))!= 0)
+  {
+    /* get the error message */
+    l=regerror(i,&cfg->validnames,NULL,0);
+    value=malloc(l);
+    if (value==NULL)
+      log_log(LOG_ERR,"%s:%d: invalid regular expression",filename,lnr);
+    else
+    {
+      regerror(i,&cfg->validnames,value,l);
+      log_log(LOG_ERR,"%s:%d: invalid regular expression: %s",filename,lnr,
+              value);
+    }
+    exit(EXIT_FAILURE);
+  }
+}
+
 static void parse_base_statement(const char *filename,int lnr,
                                  const char *keyword,char *line,
                                  struct ldap_config *cfg)
@@ -722,6 +762,39 @@ static void parse_nss_initgroups_ignoreusers_statement(
   }
 }
 
+static void parse_pam_authz_search_statement(
+              const char *filename,int lnr,const char *keyword,
+              char *line,struct ldap_config *cfg)
+{
+  SET *set;
+  const char **list;
+  int i;
+  check_argumentcount(filename,lnr,keyword,(line!=NULL)&&(*line!='\0'));
+  cfg->ldc_pam_authz_search=xstrdup(line);
+  /* check the variables used in the expression */
+  set=expr_vars(cfg->ldc_pam_authz_search,NULL);
+  list=set_tolist(set);
+  for (i=0;list[i]!=NULL;i++)
+  {
+    if ((strcmp(list[i],"username")!=0)&&
+        (strcmp(list[i],"service")!=0)&&
+        (strcmp(list[i],"ruser")!=0)&&
+        (strcmp(list[i],"rhost")!=0)&&
+        (strcmp(list[i],"tty")!=0)&&
+        (strcmp(list[i],"hostname")!=0)&&
+        (strcmp(list[i],"fqdn")!=0)&&
+        (strcmp(list[i],"dn")!=0)&&
+        (strcmp(list[i],"uid")!=0))
+    {
+      log_log(LOG_ERR,"%s:%d: unknown variable $%s",filename,lnr,list[i]);
+      exit(EXIT_FAILURE);
+    }
+  }
+  /* free memory */
+  set_free(set);
+  free(list);
+}
+
 static void cfg_read(const char *filename,struct ldap_config *cfg)
 {
   FILE *fp;
@@ -788,7 +861,16 @@ static void cfg_read(const char *filename,struct ldap_config *cfg)
         if (strcasecmp(token,"dns")==0)
         {
 #ifdef HAVE_LDAP_DOMAIN2HOSTLIST
-          add_uris_from_dns(filename,lnr,cfg);
+          add_uris_from_dns(filename,lnr,cfg,cfg_getdomainname(filename,lnr));
+#else /* not HAVE_LDAP_DOMAIN2HOSTLIST */
+          log_log(LOG_ERR,"%s:%d: value %s not supported on platform",filename,lnr,token);
+          exit(EXIT_FAILURE);
+#endif /* not HAVE_LDAP_DOMAIN2HOSTLIST */
+        }
+        else if (strncasecmp(token,"dns:",4)==0)
+        {
+#ifdef HAVE_LDAP_DOMAIN2HOSTLIST
+          add_uris_from_dns(filename,lnr,cfg,strdup(token+sizeof("dns")));
 #else /* not HAVE_LDAP_DOMAIN2HOSTLIST */
           log_log(LOG_ERR,"%s:%d: value %s not supported on platform",filename,lnr,token);
           exit(EXIT_FAILURE);
@@ -809,12 +891,18 @@ static void cfg_read(const char *filename,struct ldap_config *cfg)
     }
     else if (strcasecmp(keyword,"bindpw")==0)
     {
+      check_permissions(filename,keyword);
       get_restdup(filename,lnr,keyword,&line,&cfg->ldc_bindpw);
     }
     else if (strcasecmp(keyword,"rootpwmoddn")==0)
     {
       get_restdup(filename,lnr,keyword,&line,&cfg->ldc_rootpwmoddn);
     }
+    else if (strcasecmp(keyword,"rootpwmodpw")==0)
+    {
+      check_permissions(filename,keyword);
+      get_restdup(filename,lnr,keyword,&line,&cfg->ldc_rootpwmodpw);
+    }
     /* SASL authentication options */
     else if (strcasecmp(keyword,"use_sasl")==0)
     {
@@ -954,7 +1042,8 @@ static void cfg_read(const char *filename,struct ldap_config *cfg)
       LDAP_SET_OPTION(NULL,LDAP_OPT_X_TLS_CACERTDIR,value);
       free(value);
     }
-    else if (strcasecmp(keyword,"tls_cacertfile")==0)
+    else if ( (strcasecmp(keyword,"tls_cacertfile")==0) ||
+              (strcasecmp(keyword,"tls_cacert")==0) )
     {
       get_strdup(filename,lnr,keyword,&line,&value);
       get_eol(filename,lnr,keyword,&line);
@@ -1016,8 +1105,16 @@ static void cfg_read(const char *filename,struct ldap_config *cfg)
     }
     else if (strcasecmp(keyword,"pam_authz_search")==0)
     {
-      check_argumentcount(filename,lnr,keyword,(line!=NULL)&&(*line!='\0'));
-      cfg->ldc_pam_authz_search=xstrdup(line);
+      parse_pam_authz_search_statement(filename,lnr,keyword,line,cfg);
+    }
+    else if (strcasecmp(keyword,"nss_min_uid")==0)
+    {
+      get_uid(filename,lnr,keyword,&line,&cfg->ldc_nss_min_uid);
+      get_eol(filename,lnr,keyword,&line);
+    }
+    else if (strcasecmp(keyword,"validnames")==0)
+    {
+      parse_validnames_statement(filename,lnr,keyword,line,cfg);
     }
 #ifdef ENABLE_CONFIGFILE_CHECKING
     /* fallthrough */
@@ -1053,6 +1150,8 @@ static void bindpw_read(const char *filename,struct ldap_config *cfg)
       exit(EXIT_FAILURE);
     }
   }
+  /* check permissions */
+  check_permissions(filename,NULL);
   /* read the first line */
   if (fgets(linebuf,sizeof(linebuf),fp)==NULL)
   {
@@ -1060,8 +1159,6 @@ static void bindpw_read(const char *filename,struct ldap_config *cfg)
     exit(EXIT_FAILURE);
   }
   /* chop the last char off and save the rest as bindpw */
-  i=strlen(linebuf);
-
   i=(int)strlen(linebuf);
   if ((i<=0)||(linebuf[i-1]!='\n'))
   {
