@@ -5,7 +5,7 @@
 
    Copyright (C) 1997-2006 Luke Howard
    Copyright (C) 2006, 2007 West Consulting
-   Copyright (C) 2006, 2007, 2008, 2009, 2010 Arthur de Jong
+   Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011 Arthur de Jong
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -35,12 +35,6 @@
 
 #include "config.h"
 
-/* also include deprecated LDAP functions for now */
-#define LDAP_DEPRECATED 1
-
-/* for compatibility on Solaris */
-#define LDAP_REFERRALS 1
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -62,10 +56,10 @@
 #ifdef HAVE_GSSSASL_H
 #include <gsssasl.h>
 #endif
-/* Try to handle systems with both SASL libraries installed */
-#if defined(HAVE_SASL_SASL_H) && defined(HAVE_SASL_AUXPROP_REQUEST)
+#ifdef HAVE_SASL_SASL_H
 #include <sasl/sasl.h>
-#elif defined(HAVE_SASL_H)
+#endif
+#ifdef HAVE_SASL_H
 #include <sasl.h>
 #endif
 #include <ctype.h>
@@ -130,7 +124,7 @@ struct myldap_search
 
 /* The maximum number of ranged attribute values that may be stoted
    per entry. */
-#define MAX_RANGED_ATTRIBUTES_PER_ENTRY 2
+#define MAX_RANGED_ATTRIBUTES_PER_ENTRY 8
 
 /* A single entry from the LDAP database as returned by
    myldap_get_entry(). */
@@ -372,12 +366,12 @@ static int do_sasl_interact(LDAP UNUSED(*ld),unsigned UNUSED(flags),void *defaul
   }
 
 /* This function performs the authentication phase of opening a connection.
-   This returns an LDAP result code. */
-static int do_bind(MYLDAP_SESSION *session,const char *uri)
+   The binddn and bindpw parameters may be used to override the authentication
+   mechanism defined in the configuration.  This returns an LDAP result
+   code. */
+static int do_bind(LDAP *ld,const char *binddn,const char *bindpw,const char *uri)
 {
-#ifdef LDAP_OPT_X_TLS
   int rc;
-#endif /* LDAP_OPT_X_TLS */
 #ifdef HAVE_LDAP_SASL_INTERACTIVE_BIND_S
 #ifndef HAVE_SASL_INTERACT_T
   struct berval cred;
@@ -389,7 +383,7 @@ static int do_bind(MYLDAP_SESSION *session,const char *uri)
   {
     log_log(LOG_DEBUG,"ldap_start_tls_s()");
     errno=0;
-    rc=ldap_start_tls_s(session->ld,NULL,NULL);
+    rc=ldap_start_tls_s(ld,NULL,NULL);
     if (rc!=LDAP_SUCCESS)
     {
       log_log(LOG_WARNING,"ldap_start_tls_s() failed: %s%s%s (uri=\"%s\")",
@@ -400,12 +394,12 @@ static int do_bind(MYLDAP_SESSION *session,const char *uri)
   }
 #endif /* LDAP_OPT_X_TLS */
   /* check if the binddn and bindpw are overwritten in the session */
-  if (session->binddn[0]!='\0')
+  if ((binddn!=NULL)&(binddn[0]!='\0'))
   {
     /* do a simple bind */
-    log_log(LOG_DEBUG,"ldap_simple_bind_s(\"%s\",%s) (uri=\"%s\")",session->binddn,
-                      (session->bindpw[0]!='\0')?"\"***\"":"\"\"",uri);
-    return ldap_simple_bind_s(session->ld,session->binddn,session->bindpw);
+    log_log(LOG_DEBUG,"ldap_simple_bind_s(\"%s\",%s) (uri=\"%s\")",binddn,
+                      ((bindpw!=NULL)&&(bindpw[0]!='\0'))?"\"***\"":"\"\"",uri);
+    return ldap_simple_bind_s(ld,binddn,bindpw);
   }
   /* perform SASL bind if requested and available on platform */
 #ifdef HAVE_LDAP_SASL_INTERACTIVE_BIND_S
@@ -416,7 +410,7 @@ static int do_bind(MYLDAP_SESSION *session,const char *uri)
     if (nslcd_cfg->ldc_sasl_secprops!=NULL)
     {
       log_log(LOG_DEBUG,"ldap_set_option(LDAP_OPT_X_SASL_SECPROPS,\"%s\")",nslcd_cfg->ldc_sasl_secprops);
-      LDAP_SET_OPTION(session->ld,LDAP_OPT_X_SASL_SECPROPS,(void *)nslcd_cfg->ldc_sasl_secprops);
+      LDAP_SET_OPTION(ld,LDAP_OPT_X_SASL_SECPROPS,(void *)nslcd_cfg->ldc_sasl_secprops);
     }
 #ifdef HAVE_SASL_INTERACT_T
     if (nslcd_cfg->ldc_binddn!=NULL)
@@ -425,7 +419,7 @@ static int do_bind(MYLDAP_SESSION *session,const char *uri)
     else
       log_log(LOG_DEBUG,"ldap_sasl_interactive_bind_s(NULL,\"%s\") (uri=\"%s\")",
             nslcd_cfg->ldc_sasl_mech,uri);
-    return ldap_sasl_interactive_bind_s(session->ld,nslcd_cfg->ldc_binddn,nslcd_cfg->ldc_sasl_mech,NULL,NULL,
+    return ldap_sasl_interactive_bind_s(ld,nslcd_cfg->ldc_binddn,nslcd_cfg->ldc_sasl_mech,NULL,NULL,
                                         LDAP_SASL_QUIET,
                                         do_sasl_interact,(void *)nslcd_cfg);
 #else /* HAVE_SASL_INTERACT_T */
@@ -447,7 +441,7 @@ static int do_bind(MYLDAP_SESSION *session,const char *uri)
       log_log(LOG_DEBUG,"ldap_sasl_bind_s(NULL,\"%s\",%s) (uri=\"%s\")",
             nslcd_cfg->ldc_sasl_mech,
             nslcd_cfg->ldc_bindpw?"\"***\"":"NULL",uri);
-    return ldap_sasl_bind_s(session->ld,nslcd_cfg->ldc_binddn,nslcd_cfg->ldc_sasl_mech,&cred,NULL,NULL,NULL);
+    return ldap_sasl_bind_s(ld,nslcd_cfg->ldc_binddn,nslcd_cfg->ldc_sasl_mech,&cred,NULL,NULL,NULL);
 #endif /* not HAVE_SASL_INTERACT_T */
   }
 #endif /* HAVE_LDAP_SASL_INTERACTIVE_BIND_S */
@@ -458,21 +452,77 @@ static int do_bind(MYLDAP_SESSION *session,const char *uri)
   else
     log_log(LOG_DEBUG,"ldap_simple_bind_s(NULL,%s) (uri=\"%s\")",
                       nslcd_cfg->ldc_bindpw?"\"***\"":"NULL",uri);
-  return ldap_simple_bind_s(session->ld,nslcd_cfg->ldc_binddn,nslcd_cfg->ldc_bindpw);
+  return ldap_simple_bind_s(ld,nslcd_cfg->ldc_binddn,nslcd_cfg->ldc_bindpw);
 }
 
 #ifdef HAVE_LDAP_SET_REBIND_PROC
 /* This function is called by the LDAP library when chasing referrals.
    It is configured with the ldap_set_rebind_proc() below. */
-static int do_rebind(LDAP *UNUSED(ld),LDAP_CONST char *url,
+static int do_rebind(LDAP *ld,LDAP_CONST char *url,
                      ber_tag_t UNUSED(request),
                      ber_int_t UNUSED(msgid),void *arg)
 {
+  MYLDAP_SESSION *session=(MYLDAP_SESSION *)arg;
   log_log(LOG_DEBUG,"rebinding to %s",url);
-  return do_bind((MYLDAP_SESSION *)arg,url);
+  return do_bind(ld,session->binddn,session->bindpw,url);
 }
 #endif /* HAVE_LDAP_SET_REBIND_PROC */
 
+/* set a recieve and send timeout on a socket */
+static int set_socket_timeout(LDAP *ld,time_t sec,suseconds_t usec)
+{
+  struct timeval tv;
+  int rc=LDAP_SUCCESS;
+  int sd;
+  log_log(LOG_DEBUG,"set_socket_timeout(%lu,%lu)",sec,usec);
+  /* get the socket */
+  if ((rc=ldap_get_option(ld,LDAP_OPT_DESC,&sd))!=LDAP_SUCCESS)
+  {
+    log_log(LOG_ERR,"ldap_get_option(LDAP_OPT_DESC) failed: %s",ldap_err2string(rc));
+    return rc;
+  }
+  /* ignore invalid (probably closed) file descriptors */
+  if (sd<=0)
+    return LDAP_SUCCESS;
+  /* set timeouts */
+  memset(&tv,0,sizeof(tv));
+  tv.tv_sec=sec;
+  tv.tv_usec=usec;
+  if (setsockopt(sd,SOL_SOCKET,SO_RCVTIMEO,(void *)&tv,sizeof(tv)))
+  {
+    log_log(LOG_ERR,"setsockopt(%d,SO_RCVTIMEO) failed: %s",sd,strerror(errno));
+    rc=LDAP_LOCAL_ERROR;
+  }
+  if (setsockopt(sd,SOL_SOCKET,SO_SNDTIMEO,(void *)&tv,sizeof(tv)))
+  {
+    log_log(LOG_ERR,"setsockopt(%d,SO_RCVTIMEO) failed: %s",sd,strerror(errno));
+    rc=LDAP_LOCAL_ERROR;
+  }
+  return rc;
+}
+
+#ifdef LDAP_OPT_CONNECT_CB
+/* This function is called by the LDAP library once a connection was made to the server. We
+   set a timeout on the socket here, to catch netzwork timeouts during the ssl
+   handshake phase. It is configured with LDAP_OPT_CONNECT_CB. */
+static int connect_cb(LDAP *ld,Sockbuf UNUSED(*sb),LDAPURLDesc UNUSED(*srv),
+        struct sockaddr UNUSED(*addr),struct ldap_conncb UNUSED(*ctx))
+{
+  /* set timeout options on socket to avoid hang in some cases (a little
+     more than the normal timeout so this should only be triggered in cases
+     where the library behaves incorrectly) */
+  if (nslcd_cfg->ldc_timelimit)
+    set_socket_timeout(ld,nslcd_cfg->ldc_timelimit,500000);
+  return LDAP_SUCCESS;
+}
+
+/* We have an empty disconnect callback because LDAP_OPT_CONNECT_CB expects
+   both functions to be available. */
+static void disconnect_cb(LDAP UNUSED(*ld),Sockbuf UNUSED(*sb),struct ldap_conncb UNUSED(*ctx))
+{
+}
+#endif /* LDAP_OPT_CONNECT_CB */
+
 /* This function sets a number of properties on the connection, based
    what is configured in the configfile. This function returns an
    LDAP status code. */
@@ -481,6 +531,10 @@ static int do_set_options(MYLDAP_SESSION *session)
   /* FIXME: move this to a global initialisation routine */
   int rc;
   struct timeval tv;
+#ifdef LDAP_OPT_CONNECT_CB
+  /* make this static because OpenLDAP doesn't make it's own copy */
+  static struct ldap_conncb cb;
+#endif /* LDAP_OPT_CONNECT_CB */
 #ifdef LDAP_OPT_X_TLS
   int i;
 #endif /* LDAP_OPT_X_TLS */
@@ -527,6 +581,13 @@ static int do_set_options(MYLDAP_SESSION *session)
   LDAP_SET_OPTION(session->ld,LDAP_OPT_REFERRALS,nslcd_cfg->ldc_referrals?LDAP_OPT_ON:LDAP_OPT_OFF);
   log_log(LOG_DEBUG,"ldap_set_option(LDAP_OPT_RESTART,%s)",nslcd_cfg->ldc_restart?"LDAP_OPT_ON":"LDAP_OPT_OFF");
   LDAP_SET_OPTION(session->ld,LDAP_OPT_RESTART,nslcd_cfg->ldc_restart?LDAP_OPT_ON:LDAP_OPT_OFF);
+#ifdef LDAP_OPT_CONNECT_CB
+  /* register a connection callback */
+  cb.lc_add=connect_cb;
+  cb.lc_del=disconnect_cb;
+  cb.lc_arg=NULL;
+  LDAP_SET_OPTION(session->ld,LDAP_OPT_CONNECT_CB,(void *)&cb);
+#endif /* LDAP_OPT_CONNECT_CB */
 #ifdef LDAP_OPT_X_TLS
   /* if SSL is desired, then enable it */
   if ( (nslcd_cfg->ldc_ssl_on==SSL_LDAPS) ||
@@ -547,23 +608,19 @@ static void do_close(MYLDAP_SESSION *session)
 {
   int i;
   int rc;
-  int sd=-1;
-  struct timeval tv;
-  /* set timeout options on socket to avoid hang in some cases
-     (we set a short timeout because we don't care too much about properly
-     shutting down the connection) */
-  if (ldap_get_option(session->ld,LDAP_OPT_DESC,&sd)==LDAP_SUCCESS)
-  {
-    /* ignore errors */
-    tv.tv_sec=nslcd_cfg->ldc_timelimit/2;
-    if (!tv.tv_sec) tv.tv_sec=1;
-    tv.tv_usec=0;
-    (void)setsockopt(sd,SOL_SOCKET,SO_RCVTIMEO,(void *)&tv,sizeof(tv));
-    (void)setsockopt(sd,SOL_SOCKET,SO_SNDTIMEO,(void *)&tv,sizeof(tv));
-  }
+  time_t sec;
   /* if we had reachability problems with the server close the connection */
   if (session->ld!=NULL)
   {
+    /* set timeout options on socket to avoid hang in some cases
+       (we set a short timeout because we don't care too much about properly
+       shutting down the connection) */
+    if (nslcd_cfg->ldc_timelimit)
+    {
+      sec=nslcd_cfg->ldc_timelimit/2;
+      if (!sec) sec=1;
+      set_socket_timeout(session->ld,sec,0);
+    }
     /* go over the other searches and partially close them */
     for (i=0;i<MAX_SEARCHES_IN_SESSION;i++)
     {
@@ -600,10 +657,7 @@ static void do_close(MYLDAP_SESSION *session)
   }
 }
 
-/* This checks the timeout value of the session and closes the connection
-   to the LDAP server if the timeout has expired and there are no pending
-   searches. */
-static void myldap_session_check(MYLDAP_SESSION *session)
+void myldap_session_check(MYLDAP_SESSION *session)
 {
   int i;
   time_t current_time;
@@ -636,8 +690,6 @@ static void myldap_session_check(MYLDAP_SESSION *session)
 static int do_open(MYLDAP_SESSION *session)
 {
   int rc;
-  int sd=-1;
-  int off=0;
   /* if the connection is still there (ie. ldap_unbind() wasn't
      called) then we can return the cached connection */
   if (session->ld!=NULL)
@@ -673,7 +725,8 @@ static int do_open(MYLDAP_SESSION *session)
   }
   /* bind to the server */
   errno=0;
-  rc=do_bind(session,nslcd_cfg->ldc_uris[session->current_uri].uri);
+  rc=do_bind(session->ld,session->binddn,session->bindpw,
+             nslcd_cfg->ldc_uris[session->current_uri].uri);
   if (rc!=LDAP_SUCCESS)
   {
     /* log actual LDAP error code */
@@ -685,12 +738,6 @@ static int do_open(MYLDAP_SESSION *session)
     do_close(session);
     return rc;
   }
-  /* disable keepalive on the LDAP connection socket (why?) */
-  if (ldap_get_option(session->ld,LDAP_OPT_DESC,&sd)==LDAP_SUCCESS)
-  {
-    /* ignore errors */
-    (void)setsockopt(sd,SOL_SOCKET,SO_KEEPALIVE,(void *)&off,sizeof(off));
-  }
   /* update last activity and finish off state */
   time(&(session->lastactivity));
   return LDAP_SUCCESS;
@@ -878,7 +925,7 @@ static int do_retry_search(MYLDAP_SEARCH *search)
         if ((rc==LDAP_INVALID_CREDENTIALS)||(rc==LDAP_INSUFFICIENT_ACCESS)||
             (rc==LDAP_AUTH_METHOD_NOT_SUPPORTED))
           dotry[search->session->current_uri]=0;
-        /* check whether we should try this URI again */
+        /* check when we should try this URI again */
         else if (t <= (current_uri->firstfail+nslcd_cfg->ldc_reconnect_retrytime))
         {
           t+=nslcd_cfg->ldc_reconnect_sleeptime;
@@ -897,7 +944,7 @@ static int do_retry_search(MYLDAP_SEARCH *search)
     if (nexttry>=endtime)
     {
       if (search->session->binddn[0]=='\0')
-        log_log(LOG_ERR,"no available LDAP server found");
+        log_log(LOG_ERR,"no available LDAP server found: %s",ldap_err2string(rc));
       return rc;
     }
     /* sleep between tries */
@@ -1198,7 +1245,7 @@ MYLDAP_ENTRY *myldap_get_entry(MYLDAP_SEARCH *search,int *rcp)
         }
         /* close connection on some connection problems */
         if ((rc==LDAP_UNAVAILABLE)||(rc==LDAP_SERVER_DOWN)||(rc==LDAP_SUCCESS)||
-            (rc==LDAP_TIMELIMIT_EXCEEDED)|(rc==LDAP_OPERATIONS_ERROR)||
+            (rc==LDAP_TIMELIMIT_EXCEEDED)||(rc==LDAP_OPERATIONS_ERROR)||
             (rc==LDAP_PROTOCOL_ERROR))
         {
           do_close(search->session);
@@ -1216,7 +1263,6 @@ MYLDAP_ENTRY *myldap_get_entry(MYLDAP_SEARCH *search,int *rcp)
         if (rcp!=NULL)
           *rcp=rc;
         return NULL;
-
     }
   }
 }
@@ -1427,6 +1473,109 @@ const char **myldap_get_values(MYLDAP_ENTRY *entry,const char *attr)
   return NULL;
 }
 
+/* Convert the bervalues to a simple list of strings that can be freed
+   with one call to free(). */
+static const char **bervalues_to_values(struct berval **bvalues)
+{
+  int num_values;
+  int i;
+  size_t sz;
+  char *buf;
+  char **values;
+  /* figure out how much memory to allocate */
+  num_values=ldap_count_values_len(bvalues);
+  sz=(num_values+1)*sizeof(char *);
+  for (i=0;i<num_values;i++)
+    sz+=bvalues[i]->bv_len+1;
+  /* allocate the needed memory */
+  buf=(char *)malloc(sz);
+  if (buf==NULL)
+  {
+    log_log(LOG_CRIT,"myldap_get_values_len(): malloc() failed to allocate memory");
+    ldap_value_free_len(bvalues);
+    return NULL;
+  }
+  values=(char **)buf;
+  buf+=(num_values+1)*sizeof(char *);
+  /* copy from bvalues */
+  for (i=0;i<num_values;i++)
+  {
+    values[i]=buf;
+    memcpy(values[i],bvalues[i]->bv_val,bvalues[i]->bv_len);
+    values[i][bvalues[i]->bv_len]='\0';
+    buf+=bvalues[i]->bv_len+1;
+  }
+  values[i]=NULL;
+  return (const char **)values;
+}
+
+/* Simple wrapper around ldap_get_values(). */
+const char **myldap_get_values_len(MYLDAP_ENTRY *entry,const char *attr)
+{
+  const char **values;
+  struct berval **bvalues;
+  int rc;
+  int i;
+  SET *set;
+  /* check parameters */
+  if (!is_valid_entry(entry))
+  {
+    log_log(LOG_ERR,"myldap_get_values_len(): invalid result entry passed");
+    errno=EINVAL;
+    return NULL;
+  }
+  else if (attr==NULL)
+  {
+    log_log(LOG_ERR,"myldap_get_values_len(): invalid attribute name passed");
+    errno=EINVAL;
+    return NULL;
+  }
+  if (!entry->search->valid)
+    return NULL; /* search has been stopped */
+  /* get from LDAP */
+  bvalues=ldap_get_values_len(entry->search->session->ld,entry->search->msg,attr);
+  if (bvalues==NULL)
+  {
+    if (ldap_get_option(entry->search->session->ld,LDAP_OPT_ERROR_NUMBER,&rc)!=LDAP_SUCCESS)
+      rc=LDAP_UNAVAILABLE;
+    /* ignore decoding errors as they are just nonexisting attribute values */
+    if (rc==LDAP_DECODING_ERROR)
+    {
+      rc=LDAP_SUCCESS;
+      ldap_set_option(entry->search->session->ld,LDAP_OPT_ERROR_NUMBER,&rc);
+    }
+    else if (rc==LDAP_SUCCESS)
+    {
+      /* we have a success code but no values, let's try to get ranged
+         values */
+      set=myldap_get_ranged_values(entry,attr);
+      if (set==NULL)
+        return NULL;
+      values=set_tolist(set);
+    }
+    else
+      log_log(LOG_WARNING,"myldap_get_values_len() of attribute \"%s\" on entry \"%s\" returned NULL: %s",
+                          attr,myldap_get_dn(entry),ldap_err2string(rc));
+    return NULL;
+  }
+  else
+  {
+    values=bervalues_to_values(bvalues);
+    ldap_value_free_len(bvalues);
+  }
+  /* store values entry so we can free it later on */
+  for (i=0;i<MAX_RANGED_ATTRIBUTES_PER_ENTRY;i++)
+    if (entry->rangedattributevalues[i]==NULL)
+    {
+      entry->rangedattributevalues[i]=(char **)values;
+      return values;
+    }
+  /* we found no room to store the values */
+  log_log(LOG_ERR,"myldap_get_values_len() couldn't store results, increase MAX_RANGED_ATTRIBUTES_PER_ENTRY");
+  free(values);
+  return NULL;
+}
+
 /* Go over the entries in exploded_rdn and see if any start with
    the requested attribute. Return a reference to the value part of
    the DN (does not modify exploded_rdn). */
@@ -1443,16 +1592,15 @@ static const char *find_rdn_value(char **exploded_rdn,const char *attr)
     /* check that RDN starts with attr */
     if (strncasecmp(exploded_rdn[i],attr,l)!=0)
       continue;
+    j=l;
     /* skip spaces */
-    for (j=l;isspace(exploded_rdn[i][j]);j++)
-      /* nothing here */;
+    while (isspace(exploded_rdn[i][j])) j++;
     /* ensure that we found an equals sign now */
     if (exploded_rdn[i][j]!='=')
       continue;
     j++;
     /* skip more spaces */
-    for (;isspace(exploded_rdn[i][j]);j++)
-      /* nothing here */;
+    while (isspace(exploded_rdn[i][j])) j++;
     /* ensure that we're not at the end of the string */
     if (exploded_rdn[i][j]=='\0')
       continue;
@@ -1699,3 +1847,14 @@ int myldap_passwd(
   }
   return rc;
 }
+
+int myldap_modify(MYLDAP_SESSION *session,const char *dn,LDAPMod *mods[])
+{
+  if (!is_valid_session(session)||(dn==NULL))
+  {
+    log_log(LOG_ERR,"myldap_passwd(): invalid parameter passed");
+    errno=EINVAL;
+    return LDAP_OTHER;
+  }
+  return ldap_modify_ext_s(session->ld,dn,mods,NULL,NULL);
+}
