@@ -2,7 +2,7 @@
    nslcd.c - ldap local connection daemon
 
    Copyright (C) 2006 West Consulting
-   Copyright (C) 2006, 2007, 2008, 2009, 2010 Arthur de Jong
+   Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011 Arthur de Jong
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -52,9 +52,10 @@
 #ifndef HAVE_GETOPT_LONG
 #include "compat/getopt_long.h"
 #endif /* not HAVE_GETOPT_LONG */
-#ifndef HAVE_DAEMON
 #include "compat/daemon.h"
-#endif /* not HAVE_DAEMON */
+#include <dlfcn.h>
+#include <libgen.h>
+#include <limits.h>
 
 #include "nslcd.h"
 #include "log.h"
@@ -69,7 +70,7 @@
 #define WRITEBUFFER_MINSIZE 64
 #define WRITEBUFFER_MAXSIZE 1*1024*1024
 
-/* flag to indictate if we are in debugging mode */
+/* flag to indicate if we are in debugging mode */
 static int nslcd_debugging=0;
 
 /* flag to indicate user requested the --check option */
@@ -105,12 +106,12 @@ static void display_version(FILE *fp)
 {
   fprintf(fp,"%s\n",PACKAGE_STRING);
   fprintf(fp,"Written by Luke Howard and Arthur de Jong.\n\n");
-  fprintf(fp,"Copyright (C) 1997-2009 Luke Howard, Arthur de Jong and West Consulting\n"
+  fprintf(fp,"Copyright (C) 1997-2011 Luke Howard, Arthur de Jong and West Consulting\n"
              "This is free software; see the source for copying conditions.  There is NO\n"
              "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n");
 }
 
-/* display usage information to stdout and exit(status) */
+/* display usage information */
 static void display_usage(FILE *fp,const char *program_name)
 {
   fprintf(fp,"Usage: %s [OPTION]...\n",program_name);
@@ -232,15 +233,8 @@ static const char *signame(int signum)
 /* signal handler for closing down */
 static void sigexit_handler(int signum)
 {
-  int i;
+  /* just save the signal to indicate that we're stopping */
   nslcd_exitsignal=signum;
-  /* cancel all running threads */
-  for (i=0;i<nslcd_cfg->ldc_threads;i++)
-    if (pthread_cancel(nslcd_threads[i]))
-    {
-      /* TODO: figure out if we can actually log from within a signal handler */
-      log_log(LOG_WARNING,"failed to stop thread %d (ignored): %s",i,strerror(errno));
-    }
 }
 
 /* do some cleaning up before terminating */
@@ -268,11 +262,22 @@ static void exithandler(void)
   log_log(LOG_INFO,"version %s bailing out",VERSION);
 }
 
+/* create the directory for the specified file to reside in */
+static void mkdirname(const char *filename)
+{
+  char *tmpname;
+  tmpname=strdup(filename);
+  if (tmpname==NULL) return;
+  (void)mkdir(dirname(tmpname),(mode_t)0755);
+  free(tmpname);
+}
+
 /* returns a socket ready to answer requests from the client,
    exit()s on error */
-static int create_socket(void)
+static int create_socket(const char *filename)
 {
   int sock;
+  int i;
   struct sockaddr_un addr;
   /* create a socket */
   if ( (sock=socket(PF_UNIX,SOCK_STREAM,0))<0 )
@@ -281,21 +286,37 @@ static int create_socket(void)
     exit(EXIT_FAILURE);
   }
   /* remove existing named socket */
-  if (unlink(NSLCD_SOCKET)<0)
+  if (unlink(filename)<0)
   {
-    log_log(LOG_DEBUG,"unlink() of "NSLCD_SOCKET" failed (ignored): %s",
-            strerror(errno));
+    log_log(LOG_DEBUG,"unlink() of %s failed (ignored): %s",
+            filename,strerror(errno));
   }
+  /* do not block on accept() */
+  if ((i=fcntl(sock,F_GETFL,0))<0)
+  {
+    log_log(LOG_ERR,"fctnl(F_GETFL) failed: %s",strerror(errno));
+    if (close(sock))
+      log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
+    exit(EXIT_FAILURE);
+  }
+  if (fcntl(sock,F_SETFL,i|O_NONBLOCK)<0)
+  {
+    log_log(LOG_ERR,"fctnl(F_SETFL,O_NONBLOCK) failed: %s",strerror(errno));
+    if (close(sock))
+      log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
+    exit(EXIT_FAILURE);
+  }
+  /* create the directory if needed */
+  mkdirname(filename);
   /* create socket address structure */
   memset(&addr,0,sizeof(struct sockaddr_un));
   addr.sun_family=AF_UNIX;
-  strncpy(addr.sun_path,NSLCD_SOCKET,sizeof(addr.sun_path));
+  strncpy(addr.sun_path,filename,sizeof(addr.sun_path));
   addr.sun_path[sizeof(addr.sun_path)-1]='\0';
   /* bind to the named socket */
-  if (bind(sock,(struct sockaddr *)&addr,sizeof(struct sockaddr_un)))
+  if (bind(sock,(struct sockaddr *)&addr,(sizeof(addr.sun_family)+strlen(addr.sun_path))))
   {
-    log_log(LOG_ERR,"bind() to "NSLCD_SOCKET" failed: %s",
-            strerror(errno));
+    log_log(LOG_ERR,"bind() to %s failed: %s",filename,strerror(errno));
     if (close(sock))
       log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
     exit(EXIT_FAILURE);
@@ -303,7 +324,7 @@ static int create_socket(void)
   /* close the file descriptor on exit */
   if (fcntl(sock,F_SETFD,FD_CLOEXEC)<0)
   {
-    log_log(LOG_ERR,"fctnl(F_SETFL,O_NONBLOCK) failed: %s",strerror(errno));
+    log_log(LOG_ERR,"fctnl(F_SETFL,FD_CLOEXEC) failed: %s",strerror(errno));
     if (close(sock))
       log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
     exit(EXIT_FAILURE);
@@ -313,7 +334,7 @@ static int create_socket(void)
      fchmod does not work on sockets
      http://www.opengroup.org/onlinepubs/009695399/functions/fchmod.html
      http://lkml.org/lkml/2005/5/16/11 */
-  if (chmod(NSLCD_SOCKET,(mode_t)0666))
+  if (chmod(filename,(mode_t)0666))
   {
     log_log(LOG_ERR,"chmod(0666) failed: %s",strerror(errno));
     if (close(sock))
@@ -356,11 +377,9 @@ static void handleconnection(int sock,MYLDAP_SESSION *session)
   TFILE *fp;
   int32_t action;
   struct timeval readtimeout,writetimeout;
-  uid_t uid;
-  gid_t gid;
-  pid_t pid;
-  /* indicate new connection to logging module (genrates unique id) */
-  log_newsession();
+  uid_t uid=(uid_t)-1;
+  gid_t gid=(gid_t)-1;
+  pid_t pid=(pid_t)-1;
   /* log connection */
   if (getpeercred(sock,&uid,&gid,&pid))
     log_log(LOG_DEBUG,"connection from unknown client: %s",strerror(errno));
@@ -418,14 +437,15 @@ static void handleconnection(int sock,MYLDAP_SESSION *session)
     case NSLCD_ACTION_SERVICE_BYNAME:   (void)nslcd_service_byname(fp,session); break;
     case NSLCD_ACTION_SERVICE_BYNUMBER: (void)nslcd_service_bynumber(fp,session); break;
     case NSLCD_ACTION_SERVICE_ALL:      (void)nslcd_service_all(fp,session); break;
-    case NSLCD_ACTION_SHADOW_BYNAME:    if (uid==0) (void)nslcd_shadow_byname(fp,session); break;
-    case NSLCD_ACTION_SHADOW_ALL:       if (uid==0) (void)nslcd_shadow_all(fp,session); break;
-    case NSLCD_ACTION_PAM_AUTHC:        (void)nslcd_pam_authc(fp,session); break;
+    case NSLCD_ACTION_SHADOW_BYNAME:    if (uid==0) (void)nslcd_shadow_byname(fp,session);
+                                        else log_log(LOG_DEBUG,"denied shadow request by non-root user"); break;
+    case NSLCD_ACTION_SHADOW_ALL:       if (uid==0) (void)nslcd_shadow_all(fp,session);
+                                        else log_log(LOG_DEBUG,"denied shadow request by non-root user"); break;
+    case NSLCD_ACTION_PAM_AUTHC:        (void)nslcd_pam_authc(fp,session,uid); break;
     case NSLCD_ACTION_PAM_AUTHZ:        (void)nslcd_pam_authz(fp,session); break;
     case NSLCD_ACTION_PAM_SESS_O:       (void)nslcd_pam_sess_o(fp,session); break;
     case NSLCD_ACTION_PAM_SESS_C:       (void)nslcd_pam_sess_c(fp,session); break;
-    case NSLCD_ACTION_PAM_PWMOD:        (void)nslcd_pam_pwmod(fp,session); break;
-      /* TODO: maybe only do pwmod for (suid) root users */
+    case NSLCD_ACTION_PAM_PWMOD:        (void)nslcd_pam_pwmod(fp,session,uid); break;
     default:
       log_log(LOG_WARNING,"invalid request id: %d",(int)action);
       break;
@@ -436,45 +456,6 @@ static void handleconnection(int sock,MYLDAP_SESSION *session)
   return;
 }
 
-/* accept a connection on the socket */
-static void acceptconnection(MYLDAP_SESSION *session)
-{
-  int csock;
-  int j;
-  struct sockaddr_storage addr;
-  socklen_t alen;
-  /* accept a new connection */
-  alen=(socklen_t)sizeof(struct sockaddr_storage);
-  csock=accept(nslcd_serversocket,(struct sockaddr *)&addr,&alen);
-  if (csock<0)
-  {
-    if ((errno==EINTR)||(errno==EAGAIN)||(errno==EWOULDBLOCK))
-    {
-      log_log(LOG_DEBUG,"accept() failed (ignored): %s",strerror(errno));
-      return;
-    }
-    log_log(LOG_ERR,"accept() failed: %s",strerror(errno));
-    return;
-  }
-  /* make sure O_NONBLOCK is not inherited */
-  if ((j=fcntl(csock,F_GETFL,0))<0)
-  {
-    log_log(LOG_ERR,"fctnl(F_GETFL) failed: %s",strerror(errno));
-    if (close(csock))
-      log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
-    return;
-  }
-  if (fcntl(csock,F_SETFL,j&~O_NONBLOCK)<0)
-  {
-    log_log(LOG_ERR,"fctnl(F_SETFL,~O_NONBLOCK) failed: %s",strerror(errno));
-    if (close(csock))
-      log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
-    return;
-  }
-  /* handle the connection */
-  handleconnection(csock,session);
-}
-
 /* test to see if we can lock the specified file */
 static int is_locked(const char* filename)
 {
@@ -506,6 +487,7 @@ static void create_pidfile(const char *filename)
   char buffer[20];
   if (filename!=NULL)
   {
+    mkdirname(filename);
     if ((fd=open(filename,O_RDWR|O_CREAT,0644))<0)
     {
       log_log(LOG_ERR,"cannot create pid file (%s): %s",filename,strerror(errno));
@@ -516,7 +498,11 @@ static void create_pidfile(const char *filename)
       log_log(LOG_ERR,"cannot lock pid file (%s): %s",filename,strerror(errno));
       exit(EXIT_FAILURE);
     }
-    ftruncate(fd,0);
+    if (ftruncate(fd,0)<0)
+    {
+      log_log(LOG_ERR,"cannot truncate pid file (%s): %s",filename,strerror(errno));
+      exit(EXIT_FAILURE);
+    }
     mysnprintf(buffer,sizeof(buffer),"%d\n",(int)getpid());
     if (write(fd,buffer,strlen(buffer))!=(int)strlen(buffer))
     {
@@ -551,6 +537,12 @@ static void worker_cleanup(void *arg)
 static void *worker(void UNUSED(*arg))
 {
   MYLDAP_SESSION *session;
+  int csock;
+  int j;
+  struct sockaddr_storage addr;
+  socklen_t alen;
+  fd_set fds;
+  struct timeval tv;
   /* create a new LDAP session */
   session=myldap_create_session();
   /* clean up the session if we're done */
@@ -558,17 +550,115 @@ static void *worker(void UNUSED(*arg))
   /* start waiting for incoming connections */
   while (1)
   {
+    /* time out connection to LDAP server if needed */
+    myldap_session_check(session);
+    /* set up the set of fds to wait on */
+    FD_ZERO(&fds);
+    FD_SET(nslcd_serversocket,&fds);
+    /* set up our timeout value */
+    tv.tv_sec=nslcd_cfg->ldc_idle_timelimit;
+    tv.tv_usec=0;
     /* wait for a new connection */
-    acceptconnection(session);
+    j=select(nslcd_serversocket+1,&fds,NULL,NULL,nslcd_cfg->ldc_idle_timelimit>0?&tv:NULL);
+    /* check result of select() */
+    if (j<0)
+    {
+      if (errno==EINTR)
+        log_log(LOG_DEBUG,"debug: select() failed (ignored): %s",strerror(errno));
+      else
+        log_log(LOG_ERR,"select() failed: %s",strerror(errno));
+      continue;
+    }
+    /* see if our file descriptor is actually ready */
+    if (!FD_ISSET(nslcd_serversocket,&fds))
+      continue;
+    /* wait for a new connection */
+    alen=(socklen_t)sizeof(struct sockaddr_storage);
+    csock=accept(nslcd_serversocket,(struct sockaddr *)&addr,&alen);
+    if (csock<0)
+    {
+      if ((errno==EINTR)||(errno==EAGAIN)||(errno==EWOULDBLOCK))
+        log_log(LOG_DEBUG,"accept() failed (ignored): %s",strerror(errno));
+      else
+        log_log(LOG_ERR,"accept() failed: %s",strerror(errno));
+      continue;
+    }
+    /* make sure O_NONBLOCK is not inherited */
+    if ((j=fcntl(csock,F_GETFL,0))<0)
+    {
+      log_log(LOG_ERR,"fctnl(F_GETFL) failed: %s",strerror(errno));
+      if (close(csock))
+        log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
+      continue;
+    }
+    if (fcntl(csock,F_SETFL,j&~O_NONBLOCK)<0)
+    {
+      log_log(LOG_ERR,"fctnl(F_SETFL,~O_NONBLOCK) failed: %s",strerror(errno));
+      if (close(csock))
+        log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
+      continue;
+    }
+    /* indicate new connection to logging module (genrates unique id) */
+    log_newsession();
+    /* handle the connection */
+    handleconnection(csock,session);
+    /* indicate end of session in log messages */
+    log_clearsession();
   }
   pthread_cleanup_pop(1);
   return NULL;
 }
 
+/* function to disable lookups through the nss_ldap module to avoid lookup
+   loops */
+static void disable_nss_ldap(void)
+{
+  void *handle;
+  char *error;
+  int *enable_flag;
+  /* try to load the NSS module */
+#ifdef RTLD_NODELETE
+  handle=dlopen(NSS_LDAP_SONAME,RTLD_LAZY|RTLD_NODELETE);
+#else /* not RTLD_NODELETE */
+  handle=dlopen(NSS_LDAP_SONAME,RTLD_LAZY);
+#endif /* RTLD_NODELETE */
+  if (handle==NULL)
+  {
+    log_log(LOG_WARNING,"Warning: LDAP NSS module not loaded: %s",dlerror());
+    return;
+  }
+  /* clear any existing errors */
+  dlerror();
+  /* try to look up the flag */
+  enable_flag=(int *)dlsym(handle,"_nss_ldap_enablelookups");
+  error=dlerror();
+  if (error!=NULL)
+  {
+    log_log(LOG_WARNING,"Warning: %s (probably older NSS module loaded)",error);
+    /* fall back to changing the way host lookup is done */
+#ifdef HAVE___NSS_CONFIGURE_LOOKUP
+    if (__nss_configure_lookup("hosts","files dns"))
+      log_log(LOG_ERR,"unable to override hosts lookup method: %s",strerror(errno));
+#endif /* HAVE___NSS_CONFIGURE_LOOKUP */
+    dlclose(handle);
+    return;
+  }
+  /* disable nss_ldap */
+  *enable_flag=0;
+#ifdef RTLD_NODELETE
+  /* only close the handle if RTLD_NODELETE was used */
+  dlclose(handle);
+#endif /* RTLD_NODELETE */
+}
+
 /* the main program... */
 int main(int argc,char *argv[])
 {
   int i;
+  sigset_t signalmask,oldmask;
+#ifdef HAVE_PTHREAD_TIMEDJOIN_NP
+  struct timespec ts;
+#endif /* HAVE_PTHREAD_TIMEDJOIN_NP */
   /* parse the command line */
   parse_cmdline(argc,argv);
   /* clean the environment */
@@ -585,16 +675,8 @@ int main(int argc,char *argv[])
   /* this is a bit ugly */
   environ=sane_environment;
 #endif /* not HAVE_CLEARENV */
-  /* check if we are already running */
-  /* FIXME: implement (maybe pass along options or commands) */
-  /* disable ldap lookups of host names to avoid lookup loop
-     and fall back to files dns (a sensible default) */
-  /* TODO: parse /etc/nsswitch ourselves and just remove ldap from the list */
-#ifdef HAVE___NSS_CONFIGURE_LOOKUP
-  if (__nss_configure_lookup("hosts","files dns"))
-    log_log(LOG_ERR,"unable to override hosts lookup method: %s",strerror(errno));
-#endif /* HAVE___NSS_CONFIGURE_LOOKUP */
-  /* FIXME: have some other mechanism for systems that don't have this */
+  /* disable the nss_ldap module for this process */
+  disable_nss_ldap();
   /* set LDAP log level */
   if (myldap_set_debuglevel(nslcd_debugging)!=LDAP_SUCCESS)
     exit(EXIT_FAILURE);
@@ -624,6 +706,14 @@ int main(int argc,char *argv[])
     log_log(LOG_ERR,"daemon may already be active, cannot acquire lock (%s): %s",NSLCD_PIDFILE,strerror(errno));
     exit(EXIT_FAILURE);
   }
+  /* close all file descriptors (except stdin/out/err) */
+  i=sysconf(_SC_OPEN_MAX);
+  /* if the system does not have OPEN_MAX just close the first 32 and
+     hope we closed enough */
+  if (i<0)
+    i=32;
+  for (;i>3;i--)
+    close(i);
   /* daemonize */
   if ((!nslcd_debugging)&&(daemon(0,0)<0))
   {
@@ -643,7 +733,7 @@ int main(int argc,char *argv[])
     exit(EXIT_FAILURE);
   }
   /* create socket */
-  nslcd_serversocket=create_socket();
+  nslcd_serversocket=create_socket(NSLCD_SOCKET);
 #ifdef HAVE_SETGROUPS
   /* drop all supplemental groups */
   if (setgroups(0,NULL)<0)
@@ -673,18 +763,19 @@ int main(int argc,char *argv[])
     }
     log_log(LOG_DEBUG,"setuid(%d) done",(int)nslcd_cfg->ldc_uid);
   }
-  /* install signalhandlers for some signals */
-  install_sighandler(SIGHUP, sigexit_handler);
-  install_sighandler(SIGINT, sigexit_handler);
-  install_sighandler(SIGQUIT,sigexit_handler);
-  install_sighandler(SIGABRT,sigexit_handler);
-  install_sighandler(SIGPIPE,SIG_IGN);
-  install_sighandler(SIGTERM,sigexit_handler);
-  install_sighandler(SIGUSR1,sigexit_handler);
-  install_sighandler(SIGUSR2,sigexit_handler);
-  /* TODO: install signal handlers for reloading configuration */
-  log_log(LOG_INFO,"accepting connections");
+  /* block all these signals so our worker threads won't handle them */
+  sigemptyset(&signalmask);
+  sigaddset(&signalmask,SIGHUP);
+  sigaddset(&signalmask,SIGINT);
+  sigaddset(&signalmask,SIGQUIT);
+  sigaddset(&signalmask,SIGABRT);
+  sigaddset(&signalmask,SIGPIPE);
+  sigaddset(&signalmask,SIGTERM);
+  sigaddset(&signalmask,SIGUSR1);
+  sigaddset(&signalmask,SIGUSR2);
+  pthread_sigmask(SIG_BLOCK,&signalmask,&oldmask);
   /* start worker threads */
+  log_log(LOG_INFO,"accepting connections");
   nslcd_threads=(pthread_t *)malloc(nslcd_cfg->ldc_threads*sizeof(pthread_t));
   if (nslcd_threads==NULL)
   {
@@ -699,27 +790,44 @@ int main(int argc,char *argv[])
       exit(EXIT_FAILURE);
     }
   }
-  /* wait for all threads to die */
-  /* BUG: this causes problems if for some reason we want to exit but one
-          of our threads hangs (e.g. has one of the LDAP locks)
-     Other than that it may be a good idea to keep this thread more or less alive
-     to do general house keeping things (e.g. checking signals etc) */
-  /* it is also better to always do thread_cancel() here instead of in the signal
-     handler */
+  pthread_sigmask(SIG_SETMASK,&oldmask,NULL);
+  /* install signalhandlers for some signals */
+  install_sighandler(SIGHUP, sigexit_handler);
+  install_sighandler(SIGINT, sigexit_handler);
+  install_sighandler(SIGQUIT,sigexit_handler);
+  install_sighandler(SIGABRT,sigexit_handler);
+  install_sighandler(SIGPIPE,SIG_IGN);
+  install_sighandler(SIGTERM,sigexit_handler);
+  install_sighandler(SIGUSR1,sigexit_handler);
+  install_sighandler(SIGUSR2,sigexit_handler);
+  /* wait until we received a signal */
+  while (nslcd_exitsignal==0)
+  {
+    sleep(INT_MAX); /* sleep as long as we can or until we receive a signal */
+  }
+  /* print something about received signal */
+  log_log(LOG_INFO,"caught signal %s (%d), shutting down",
+               signame(nslcd_exitsignal),nslcd_exitsignal);
+  /* cancel all running threads */
+  for (i=0;i<nslcd_cfg->ldc_threads;i++)
+    if (pthread_cancel(nslcd_threads[i]))
+      log_log(LOG_WARNING,"failed to stop thread %d (ignored): %s",i,strerror(errno));
+  /* close server socket to trigger failures in threads waiting on accept() */
+  close(nslcd_serversocket);
+  nslcd_serversocket=-1;
+  /* if we can, wait a few seconds for the threads to finish */
+#ifdef HAVE_PTHREAD_TIMEDJOIN_NP
+  ts.tv_sec=time(NULL)+3;
+  ts.tv_nsec=0;
+#endif /* HAVE_PTHREAD_TIMEDJOIN_NP */
   for (i=0;i<nslcd_cfg->ldc_threads;i++)
   {
-    if (pthread_join(nslcd_threads[i],NULL))
-    {
-      log_log(LOG_ERR,"unable to wait for worker thread %d: %s",i,strerror(errno));
-      exit(EXIT_FAILURE);
-    }
-  }
-  free(nslcd_threads);
-  /* print something about received signals */
-  if (nslcd_exitsignal!=0)
-  {
-    log_log(LOG_INFO,"caught signal %s (%d), shutting down",
-                 signame(nslcd_exitsignal),nslcd_exitsignal);
+#ifdef HAVE_PTHREAD_TIMEDJOIN_NP
+    pthread_timedjoin_np(nslcd_threads[i],NULL,&ts);
+#endif /* HAVE_PTHREAD_TIMEDJOIN_NP */
+    if (pthread_kill(nslcd_threads[i],0)==0)
+      log_log(LOG_ERR,"thread %d is still running, shutting down anyway",i);
   }
+  /* we're done */
   return EXIT_FAILURE;
 }
