@@ -37,7 +37,32 @@ attmap = common.Attributes(cn='cn',
                            gidNumber='gidNumber',
                            memberUid='memberUid',
                            member='member')
-filter = '(|(objectClass=posixGroup)(objectClass=groupOfNames))'
+filter = '(objectClass=posixGroup)'
+
+
+class Search(common.Search):
+
+    case_sensitive = ('cn', )
+    limit_attributes = ('cn', 'gidNumber')
+
+    def __init__(self, *args, **kwargs):
+        super(Search, self).__init__(*args, **kwargs)
+        if 'memberUid' in self.parameters:
+            # set up our own attributes that leave out membership attributes
+            self.attributes = list(self.attributes)
+            self.attributes.remove(attmap['memberUid'])
+            self.attributes.remove(attmap['member'])
+
+    def mk_filter(self):
+        # we still need a custom mk_filter because this is an | query
+        if attmap['member'] and 'memberUid' in self.parameters:
+            memberuid = self.parameters['memberUid']
+            dn = uid2dn(self.conn, memberuid)
+            if dn:
+                return '(&%s(|(%s=%s)(%s=%s)))' % (self.filter,
+                          attmap['memberUid'], ldap.filter.escape_filter_chars(memberuid),
+                          attmap['member'], ldap.filter.escape_filter_chars(dn))
+        return super(Search, self).mk_filter()
 
 
 class GroupRequest(common.Request):
@@ -47,15 +72,10 @@ class GroupRequest(common.Request):
     def write(self, dn, attributes, parameters):
         # get group names and check against requested group name
         names = attributes['cn']
-        if 'cn' in parameters:
-            if parameters['cn'] not in names:
-                return
-            names = ( parameters['cn'], )
         # get group group password
         passwd = attributes['userPassword'][0]
         # get group id(s)
-        gids = (  parameters['gidNumber'], ) if 'gidNumber' in parameters else attributes['gidNumber']
-        gids = [ int(x) for x in gids ]
+        gids = [int(x) for x in attributes['gidNumber']]
         # build member list
         members = set()
         if self.wantmembers:
@@ -71,7 +91,7 @@ class GroupRequest(common.Request):
         # actually return the results
         for name in names:
             if not common.isvalidname(name):
-                print 'Warning: group entry %s contains invalid group name: "%s"' % ( dn, name )
+                print '%s: %s: denied by validnames option' % (dn, attmap['cn'])
             else:
                 for gid in gids:
                     self.fp.write_int32(constants.NSLCD_RESULT_BEGIN)
@@ -104,33 +124,11 @@ class GroupByMemberRequest(GroupRequest):
     action = constants.NSLCD_ACTION_GROUP_BYMEMBER
     wantmembers = False
 
-    def __init__(self, *args, **kwargs):
-        super(GroupByMemberRequest, self).__init__(*args, **kwargs)
-        # set up our own attributes that leave out membership attributes
-        self.attmap = common.Attributes(attmap)
-        del self.attmap['memberUid']
-        del self.attmap['member']
-
     def read_parameters(self, fp):
         memberuid = fp.read_string()
         common.validate_name(memberuid)
         return dict(memberUid=memberuid)
 
-    def attributes(self):
-        return self.attmap.attributes()
-
-    def mk_filter(self, parameters):
-        # we still need a custom mk_filter because this is an | query
-        memberuid = parameters['memberUid']
-        if attmap['member']:
-            dn = uid2dn(self.conn, memberuid)
-            if dn:
-                return '(&%s(|(%s=%s)(%s=%s)))' % ( self.filter,
-                          attmap['memberUid'], ldap.filter.escape_filter_chars(memberuid),
-                          attmap['member'], ldap.filter.escape_filter_chars(dn) )
-        return '(&%s(%s=%s))' % ( self.filter,
-                  attmap['memberUid'], ldap.filter.escape_filter_chars(memberuid) )
-
 
 class GroupAllRequest(GroupRequest):
 
